#!/usr/bin/python
"""Miscellaneous methods for MegaUpload downloader"""

# Copyright (c) 2008-2009 Arnau Sanchez <tokland@gmail.com>

# This file is part of Megaupload-dl.

# This script is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this software.  If not, see <http://www.gnu.org/licenses/>

 
import BeautifulSoup
import subprocess
import itertools
import htmllib
import urllib2
import urllib
import sys
import re
import os

def output(obj):
    """Print a debug info line to stderr"""
    sys.stdout.write(str(obj)+"\n")

def debug(obj):
    """Write obj to standard error"""
    sys.stderr.write("--- " + str(obj)+"\n")
    sys.stderr.flush()

def get_debug_func(debug_level):
    """Get a function to write debug info lines with a given debug_level"""
    def _debug(level, obj):
        if level <= debug_level:
            debug(obj)
    return _debug    
    
def first(iterable, pred=bool):
    """Return first item in iterator that matches the predicate"""
    for item in iterable:
        if pred(item):
            return item

def search(regexp, string):
    """Return matched groups if regexp in s"""
    return re.search(regexp, string).groups()
                           
def get_soup(data):
    """Return a BeautifulSoup from plain HTML data"""
    return BeautifulSoup.BeautifulSoup(data)
               
def download(url):
    """Download a URL, optionally using an opener"""
    return urllib2.urlopen(url).read()

def build_request(url, postdata=None):
    """Build a URL request with POSTDATA"""
    data = urllib.urlencode(postdata)
    return urllib2.Request(url, data)

def run(command, inputdata=None):
    """Run a command and return standard output"""
    pipe = subprocess.PIPE
    popen = subprocess.Popen(command, stdout=pipe, stderr=pipe)
    outputdata = popen.communicate(inputdata)[0]
    assert (popen.returncode == 0), "Error running: %s" % command
    return outputdata

def ocr(image):
    """OCR an image and return text"""
    tbase = "_test"
    image.save("%s.tif" % tbase)
    run(("tesseract %s.tif %s" %(tbase, tbase)).split())
    text = open("%s.txt" % tbase).read().strip()
    os.unlink("%s.tif" % tbase)
    os.unlink("%s.txt" % tbase)
    return text
    
def unescape_entities(html_string):
    """Removes HTML or XML character references and entities 
    from a html string (see http://wiki.python.org/moin/EscapingHtml)"""    
    parser = htmllib.HTMLParser(None)
    parser.save_bgn()
    parser.feed(html_string)
    return parser.save_end()
